@@ -47,7 +47,6 @@ CUSTOM_REQUEST_SENDER = None
 ENABLE_MIDDLEWARE = False
 
 
-
 def _get_req_session(reset=False):
     if SESSION_TIME_TO_LIVE:
         # If session TTL is set - check time passed
@@ -94,20 +93,14 @@ def _make_request(token, method_name, method='get', params=None, files=None):
         if 'timeout' in params:
             read_timeout = params.pop('timeout')
             connect_timeout = read_timeout
-#        if 'connect-timeout' in params:
-#            connect_timeout = params.pop('connect-timeout') + 10
         if 'long_polling_timeout' in params:
-            # For getUpdates: it's the only function with timeout parameter on the BOT API side
+            # For getUpdates. It's the only function with timeout parameter on the BOT API side
             long_polling_timeout = params.pop('long_polling_timeout')
             params['timeout'] = long_polling_timeout
             # Long polling hangs for a given time. Read timeout should be greater that long_polling_timeout
             read_timeout = max(long_polling_timeout + 5, read_timeout)
-    # Lets stop suppose that user is stupid and assume that he knows what he do...
-    # read_timeout = read_timeout + 10
-    # connect_timeout = connect_timeout + 10
 
     params = params or None # Set params to None if empty
-
     result = None
     if RETRY_ON_ERROR and RETRY_ENGINE == 1:
         got_result = False
@@ -134,6 +127,7 @@ def _make_request(token, method_name, method='get', params=None, files=None):
                     timeout=(connect_timeout, read_timeout), proxies=proxy)
     elif RETRY_ON_ERROR and RETRY_ENGINE == 2:
         http = _get_req_session()
+        # noinspection PyUnresolvedReferences
         retry_strategy = requests.packages.urllib3.util.retry.Retry(
             total=MAX_RETRIES,
         )
@@ -971,7 +965,7 @@ def promote_chat_member(
         token, chat_id, user_id, can_change_info=None, can_post_messages=None,
         can_edit_messages=None, can_delete_messages=None, can_invite_users=None,
         can_restrict_members=None, can_pin_messages=None, can_promote_members=None,
-        is_anonymous=None, can_manage_chat=None, can_manage_voice_chats=None):
+        is_anonymous=None, can_manage_chat=None, can_manage_video_chats=None):
     method_url = 'promoteChatMember'
     payload = {'chat_id': chat_id, 'user_id': user_id}
     if can_change_info is not None:
@@ -994,8 +988,8 @@ def promote_chat_member(
         payload['is_anonymous'] = is_anonymous
     if can_manage_chat is not None:
         payload['can_manage_chat'] = can_manage_chat
-    if can_manage_voice_chats is not None:
-        payload['can_manage_voice_chats'] = can_manage_voice_chats
+    if can_manage_video_chats is not None:
+        payload['can_manage_video_chats'] = can_manage_video_chats
     return _make_request(token, method_url, params=payload, method='post')
 
 
@@ -1139,6 +1133,40 @@ def get_my_commands(token, scope=None, language_code=None):
         payload['language_code'] = language_code
     return _make_request(token, method_url, params=payload)
 
+def set_chat_menu_button(token, chat_id=None, menu_button=None):
+    method_url = r'setChatMenuButton'
+    payload = {}
+    if chat_id:
+        payload['chat_id'] = chat_id
+    if menu_button:
+        payload['menu_button'] = menu_button.to_json()
+    return _make_request(token, method_url, params=payload, method='post')
+
+def get_chat_menu_button(token, chat_id=None):
+    method_url = r'getChatMenuButton'
+    payload = {}
+    if chat_id:
+        payload['chat_id'] = chat_id
+    return _make_request(token, method_url, params=payload, method='post')
+
+
+def set_my_default_administrator_rights(token, rights=None, for_channels=None):
+    method_url = r'setMyDefaultAdministratorRights'
+    payload = {}
+    if rights:
+        payload['rights'] = rights.to_json()
+    if for_channels is not None:
+        payload['for_channels'] = for_channels
+    return _make_request(token, method_url, params=payload, method='post')
+
+
+def get_my_default_administrator_rights(token, for_channels=None):
+    method_url = r'getMyDefaultAdministratorRights'
+    payload = {}
+    if for_channels:
+        payload['for_channels'] = for_channels
+    return _make_request(token, method_url, params=payload, method='post')
+
 
 def set_my_commands(token, commands, scope=None, language_code=None):
     method_url = r'setMyCommands'
@@ -1289,7 +1317,6 @@ def send_game(
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if protect_content is not None:
         payload['protect_content'] = protect_content
-    
     return _make_request(token, method_url, params=payload)
 
 
@@ -1533,11 +1560,16 @@ def upload_sticker_file(token, user_id, png_sticker):
 
 def create_new_sticker_set(
         token, user_id, name, title, emojis, png_sticker, tgs_sticker,
-        contains_masks=None, mask_position=None):
+        contains_masks=None, mask_position=None, webm_sticker=None):
     method_url = 'createNewStickerSet'
     payload = {'user_id': user_id, 'name': name, 'title': title, 'emojis': emojis}
-    stype = 'png_sticker' if png_sticker else 'tgs_sticker'
-    sticker = png_sticker or tgs_sticker
+    if png_sticker:
+        stype = 'png_sticker'
+    elif webm_sticker:
+        stype = 'webm_sticker'
+    else:
+        stype = 'tgs_sticker'
+    sticker = png_sticker or tgs_sticker or webm_sticker
     files = None
     if not util.is_string(sticker):
         files = {stype: sticker}
@@ -1547,14 +1579,21 @@ def create_new_sticker_set(
         payload['contains_masks'] = contains_masks
     if mask_position:
         payload['mask_position'] = mask_position.to_json()
+    if webm_sticker:
+        payload['webm_sticker'] = webm_sticker
     return _make_request(token, method_url, params=payload, files=files, method='post')
 
 
-def add_sticker_to_set(token, user_id, name, emojis, png_sticker, tgs_sticker, mask_position):
+def add_sticker_to_set(token, user_id, name, emojis, png_sticker, tgs_sticker, mask_position, webm_sticker):
     method_url = 'addStickerToSet'
     payload = {'user_id': user_id, 'name': name, 'emojis': emojis}
-    stype = 'png_sticker' if png_sticker else 'tgs_sticker'
-    sticker = png_sticker or tgs_sticker
+    if png_sticker:
+        stype = 'png_sticker'
+    elif webm_sticker:
+        stype = 'webm_sticker'
+    else:
+        stype = 'tgs_sticker'
+    sticker = png_sticker or tgs_sticker or webm_sticker
     files = None
     if not util.is_string(sticker):
         files = {stype: sticker}
@@ -1577,6 +1616,12 @@ def delete_sticker_from_set(token, sticker):
     return _make_request(token, method_url, params=payload, method='post')
 
 
+def answer_web_app_query(token, web_app_query_id, result: types.InlineQueryResultBase):
+    method_url = 'answerWebAppQuery'
+    payload = {'web_app_query_id': web_app_query_id, 'result': result.to_json()}
+    return _make_request(token, method_url, params=payload, method='post')
+
+
 # noinspection PyShadowingBuiltins
 def send_poll(
         token, chat_id,
@@ -1721,7 +1766,8 @@ class ApiException(Exception):
         super(ApiException, self).__init__("A request to the Telegram API was unsuccessful. {0}".format(msg))
         self.function_name = function_name
         self.result = result
-    
+
+
 class ApiHTTPException(ApiException):
     """
     This class represents an Exception thrown when a call to the 
@@ -1733,7 +1779,8 @@ class ApiHTTPException(ApiException):
             .format(result.status_code, result.reason, result.text.encode('utf8')),
             function_name,
             result)
-    
+
+
 class ApiInvalidJSONException(ApiException):
     """
     This class represents an Exception thrown when a call to the 
@@ -1745,7 +1792,8 @@ class ApiInvalidJSONException(ApiException):
             .format(result.text.encode('utf8')),
             function_name,
             result)
-    
+
+
 class ApiTelegramException(ApiException):
     """
     This class represents an Exception thrown when a Telegram API returns error code.
@@ -1759,4 +1807,3 @@ class ApiTelegramException(ApiException):
         self.result_json = result_json
         self.error_code = result_json['error_code']
         self.description = result_json['description']
-        
