@@ -11,7 +11,6 @@ API_URL = 'https://api.telegram.org/bot{0}/{1}'
 
 from datetime import datetime
 
-import telebot
 from telebot import util, logger
 
 
@@ -26,8 +25,6 @@ READ_TIMEOUT = 30
 LONG_POLLING_TIMEOUT = 10 # Should be positive, short polling should be used for testing purposes only (https://core.telegram.org/bots/api#getupdates)
 REQUEST_TIMEOUT = 10
 MAX_RETRIES = 3
-logger = telebot.logger
-
 
 REQUEST_LIMIT = 50
 
@@ -63,17 +60,18 @@ async def _process_request(token, url, method='get', params=None, files=None, re
     while not got_result and current_try<MAX_RETRIES-1:
         current_try +=1
         try:
-            async with session.request(method=method, url=API_URL.format(token, url), data=params, timeout=timeout) as resp:
+            async with session.request(method=method, url=API_URL.format(token, url), data=params, timeout=timeout, proxy=proxy) as resp:
                 logger.debug("Request: method={0} url={1} params={2} files={3} request_timeout={4} current_try={5}".format(method, url, params, files, request_timeout, current_try).replace(token, token.split(':')[0] + ":{TOKEN}"))
                 json_result = await _check_result(url, resp)
                 if json_result:
+                    got_result = True
                     return json_result['result']
         except (ApiTelegramException,ApiInvalidJSONException, ApiHTTPException) as e:
             raise e
         except aiohttp.ClientError as e:
             logger.error('Aiohttp ClientError: {0}'.format(e.__class__.__name__))
         except Exception as e:
-            logger.error(f'Unkown error: {e.__class__.__name__}')
+            logger.error(f'Unknown error: {e.__class__.__name__}')
         if not got_result:
             raise RequestTimeout("Request timeout. Request: method={0} url={1} params={2} files={3} request_timeout={4}".format(method, url, params, files, request_timeout, current_try))
         
@@ -356,6 +354,12 @@ async def delete_chat_sticker_set(token, chat_id):
     return await _process_request(token, method_url, params=payload)
 
 
+async def answer_web_app_query(token, web_app_query_id, result: types.InlineQueryResultBase):
+    method_url = 'answerWebAppQuery'
+    payload = {'web_app_query_id': web_app_query_id, 'result': result.to_json()}
+    return await _process_request(token, method_url, params=payload, method='post')
+
+
 async def get_chat_member(token, chat_id, user_id):
     method_url = r'getChatMember'
     payload = {'chat_id': chat_id, 'user_id': user_id}
@@ -940,7 +944,7 @@ async def promote_chat_member(
         token, chat_id, user_id, can_change_info=None, can_post_messages=None,
         can_edit_messages=None, can_delete_messages=None, can_invite_users=None,
         can_restrict_members=None, can_pin_messages=None, can_promote_members=None,
-        is_anonymous=None, can_manage_chat=None, can_manage_voice_chats=None):
+        is_anonymous=None, can_manage_chat=None, can_manage_video_chats=None):
     method_url = 'promoteChatMember'
     payload = {'chat_id': chat_id, 'user_id': user_id}
     if can_change_info is not None:
@@ -963,8 +967,8 @@ async def promote_chat_member(
         payload['is_anonymous'] = is_anonymous
     if can_manage_chat is not None:
         payload['can_manage_chat'] = can_manage_chat
-    if can_manage_voice_chats is not None:
-        payload['can_manage_voice_chats'] = can_manage_voice_chats
+    if can_manage_video_chats is not None:
+        payload['can_manage_video_chats'] = can_manage_video_chats
     return await _process_request(token, method_url, params=payload, method='post')
 
 
@@ -1105,6 +1109,42 @@ async def get_my_commands(token, scope=None, language_code=None):
         payload['language_code'] = language_code
     return await _process_request(token, method_url, params=payload)
 
+async def set_chat_menu_button(token, chat_id=None, menu_button=None):
+    method_url = r'setChatMenuButton'
+    payload = {}
+    if chat_id:
+        payload['chat_id'] = chat_id
+    if menu_button:
+        payload['menu_button'] = menu_button.to_json()
+
+    return await _process_request(token, method_url, params=payload, method='post')
+
+async def get_chat_menu_button(token, chat_id=None):
+    method_url = r'getChatMenuButton'
+    payload = {}
+    if chat_id:
+        payload['chat_id'] = chat_id
+
+    return await _process_request(token, method_url, params=payload, method='post')
+
+
+async def set_my_default_administrator_rights(token, rights=None, for_channels=None):
+    method_url = r'setMyDefaultAdministratorRights'
+    payload = {}
+    if rights:
+        payload['rights'] = rights.to_json()
+    if for_channels is not None:
+        payload['for_channels'] = for_channels
+
+    return await _process_request(token, method_url, params=payload, method='post')
+
+async def get_my_default_administrator_rights(token, for_channels=None):
+    method_url = r'getMyDefaultAdministratorRights'
+    payload = {}
+    if for_channels:
+        payload['for_channels'] = for_channels
+
+    return await _process_request(token, method_url, params=payload, method='post')
 
 async def set_my_commands(token, commands, scope=None, language_code=None):
     method_url = r'setMyCommands'
@@ -1200,7 +1240,7 @@ async def edit_message_caption(token, caption, chat_id=None, message_id=None, in
 
 async def edit_message_media(token, media, chat_id=None, message_id=None, inline_message_id=None, reply_markup=None):
     method_url = r'editMessageMedia'
-    media_json, file = convert_input_media(media)
+    media_json, file = await convert_input_media(media)
     payload = {'media': media_json}
     if chat_id:
         payload['chat_id'] = chat_id
@@ -1500,7 +1540,6 @@ async def create_new_sticker_set(
         contains_masks=None, mask_position=None, webm_sticker=None):
     method_url = 'createNewStickerSet'
     payload = {'user_id': user_id, 'name': name, 'title': title, 'emojis': emojis}
-    stype = None
     if png_sticker:
         stype = 'png_sticker'
     elif webm_sticker:
@@ -1525,7 +1564,6 @@ async def create_new_sticker_set(
 async def add_sticker_to_set(token, user_id, name, emojis, png_sticker, tgs_sticker, mask_position, webm_sticker):
     method_url = 'addStickerToSet'
     payload = {'user_id': user_id, 'name': name, 'emojis': emojis}
-    stype = None
     if png_sticker:
         stype = 'png_sticker'
     elif webm_sticker:
