@@ -21,6 +21,7 @@ try:
     # noinspection PyPackageRequirements
     from PIL import Image
     from io import BytesIO
+
     pil_imported = True
 except:
     pil_imported = False
@@ -39,16 +40,17 @@ content_type_media = [
 content_type_service = [
     'new_chat_members', 'left_chat_member', 'new_chat_title', 'new_chat_photo', 'delete_chat_photo', 'group_chat_created',
     'supergroup_chat_created', 'channel_chat_created', 'migrate_to_chat_id', 'migrate_from_chat_id', 'pinned_message',
-    'proximity_alert_triggered', 'voice_chat_scheduled', 'voice_chat_started', 'voice_chat_ended', 
-    'voice_chat_participants_invited', 'message_auto_delete_timer_changed'
+    'proximity_alert_triggered', 'video_chat_scheduled', 'video_chat_started', 'video_chat_ended',
+    'video_chat_participants_invited', 'message_auto_delete_timer_changed'
 ]
 
 update_types = [
-    "update_id", "message", "edited_message", "channel_post", "edited_channel_post", "inline_query", 
-    "chosen_inline_result", "callback_query", "shipping_query", "pre_checkout_query", "poll", "poll_answer", 
+    "update_id", "message", "edited_message", "channel_post", "edited_channel_post", "inline_query",
+    "chosen_inline_result", "callback_query", "shipping_query", "pre_checkout_query", "poll", "poll_answer",
     "my_chat_member", "chat_member", "chat_join_request"
 ]
 
+
 class WorkerThread(threading.Thread):
     count = 0
 
@@ -114,7 +116,8 @@ class WorkerThread(threading.Thread):
 
 class ThreadPool:
 
-    def __init__(self, num_threads=2):
+    def __init__(self, telebot, num_threads=2):
+        self.telebot = telebot
         self.tasks = Queue.Queue()
         self.workers = [WorkerThread(self.on_exception, self.tasks) for _ in range(num_threads)]
         self.num_threads = num_threads
@@ -126,8 +129,13 @@ class ThreadPool:
         self.tasks.put((func, args, kwargs))
 
     def on_exception(self, worker_thread, exc_info):
-        self.exception_info = exc_info
-        self.exception_event.set()
+        if self.telebot.exception_handler is not None:
+            handled = self.telebot.exception_handler.handle(exc_info)
+        else:
+            handled = False
+        if not handled:
+            self.exception_info = exc_info
+            self.exception_event.set()
         worker_thread.continue_event.set()
 
     def raise_exceptions(self):
@@ -211,15 +219,16 @@ def pil_image_to_file(image, extension='JPEG', quality='web_low'):
         photoBuffer = BytesIO()
         image.convert('RGB').save(photoBuffer, extension, quality=quality)
         photoBuffer.seek(0)
-        
+
         return photoBuffer
     else:
         raise RuntimeError('PIL module is not imported')
 
 
 def is_command(text: str) -> bool:
-    """
+    r"""
     Checks if `text` is a command. Telegram chat commands start with the '/' character.
+    
     :param text: Text to check.
     :return: True if `text` is a command, else False.
     """
@@ -275,7 +284,7 @@ def split_string(text: str, chars_per_string: int) -> List[str]:
 
 
 def smart_split(text: str, chars_per_string: int=MAX_MESSAGE_LENGTH) -> List[str]:
-    """
+    r"""
     Splits one string into multiple strings, with a maximum amount of `chars_per_string` characters per string.
     This is very useful for splitting one giant message into multiples.
     If `chars_per_string` > 4096: `chars_per_string` = 4096.
@@ -314,7 +323,7 @@ def escape(text: str) -> str:
     :param text: the text to escape
     :return: the escaped text
     """
-    chars = {"&": "&amp;", "<": "&lt;", ">": "&gt"}
+    chars = {"&": "&amp;", "<": "&lt;", ">": "&gt;"}
     for old, new in chars.items(): text = text.replace(old, new)
     return text
 
@@ -332,7 +341,7 @@ def user_link(user: types.User, include_id: bool=False) -> str:
     :return: HTML user link
     """
     name = escape(user.first_name)
-    return (f"<a href='tg://user?id={user.id}'>{name}</a>" 
+    return (f"<a href='tg://user?id={user.id}'>{name}</a>"
         + (f" (<pre>{user.id}</pre>)" if include_id else ""))
 
 
@@ -342,24 +351,27 @@ def quick_markup(values: Dict[str, Dict[str, Any]], row_width: int=2) -> types.I
     This is useful to avoid always typing 'btn1 = InlineKeyboardButton(...)' 'btn2 = InlineKeyboardButton(...)' 
     
     Example:
-    quick_markup({
-        'Twitter': {'url': 'https://twitter.com'},
-        'Facebook': {'url': 'https://facebook.com'},
-        'Back': {'callback_data': 'whatever'}
-    }, row_width=2): 
-        returns an InlineKeyboardMarkup with two buttons in a row, one leading to Twitter, the other to facebook
-        and a back button below
 
-    kwargs can be: 
-    {
-        'url': None, 
-        'callback_data': None, 
-        'switch_inline_query': None,
-        'switch_inline_query_current_chat': None,
-        'callback_game': None,
-        'pay': None,
-        'login_url': None
-    }
+    .. code-block:: python
+
+        quick_markup({
+            'Twitter': {'url': 'https://twitter.com'},
+            'Facebook': {'url': 'https://facebook.com'},
+            'Back': {'callback_data': 'whatever'}
+        }, row_width=2): 
+            # returns an InlineKeyboardMarkup with two buttons in a row, one leading to Twitter, the other to facebook
+            # and a back button below
+
+        # kwargs can be: 
+        {
+            'url': None, 
+            'callback_data': None, 
+            'switch_inline_query': None,
+            'switch_inline_query_current_chat': None,
+            'callback_game': None,
+            'pay': None,
+            'login_url': None
+        }
     
     :param values: a dict containing all buttons to create in this format: {text: kwargs} {str:}
     :param row_width: int row width
@@ -437,16 +449,22 @@ def generate_random_token():
     return ''.join(random.sample(string.ascii_letters, 16))
 
 
-def deprecated(warn: bool=True, alternative: Optional[Callable]=None):
+def deprecated(warn: bool=True, alternative: Optional[Callable]=None, deprecation_text=None):
     """
     Use this decorator to mark functions as deprecated.
     When the function is used, an info (or warning if `warn` is True) is logged.
+    
     :param warn: If True a warning is logged else an info
     :param alternative: The new function to use instead
+    :param deprecation_text: Custom deprecation text
     """
     def decorator(function):
         def wrapper(*args, **kwargs):
-            info = f"`{function.__name__}` is deprecated." + (f" Use `{alternative.__name__}` instead" if alternative else "")
+            info = f"`{function.__name__}` is deprecated."
+            if alternative:
+                info += f" Use `{alternative.__name__}` instead"
+            if deprecation_text:
+                info += " " + deprecation_text
             if not warn:
                 logger.info(info)
             else:
@@ -471,16 +489,22 @@ def webhook_google_functions(bot, request):
     else:
         return 'Bot ON'
 
+
 def antiflood(function, *args, **kwargs):
     """
     Use this function inside loops in order to avoid getting TooManyRequests error.
     Example:
-
-    from telebot.util import antiflood
-    for chat_id in chat_id_list:
+    
+    .. code-block:: python3
+    
+        from telebot.util import antiflood
+        for chat_id in chat_id_list:
         msg = antiflood(bot.send_message, chat_id, text)
         
-    You want get the
+    :param function:
+    :param args:
+    :param kwargs:
+    :return: None
     """
     from telebot.apihelper import ApiTelegramException
     from time import sleep
